import logging,os,threading,traceback
from contextlib import contextmanager
from typing import List
from localstack import constants
from localstack.packages import InstallTarget,PackageException
from localstack.services.install import ARTIFACTS_REPO,Installer,InstallerRepository
from localstack.utils.http import download
from localstack.utils.time import now
from localstack_ext.bootstrap.licensing import api_key_configured,is_enterprise,prepare_environment
LOG=logging.getLogger(__name__)
H2_DOWNLOAD_URL='http://www.h2database.com/h2-2019-10-14.zip'
SSL_CERT_URL=f"{ARTIFACTS_REPO}/raw/master/local-certs/server.key"
SSL_CERT_URL_FALLBACK='{api_endpoint}/proxy/localstack.cert.key'
INSTALL_LOCK=threading.RLock()
@contextmanager
def log_package_exception():
	try:yield
	except PackageException:LOG.exception('Package installation failed.')
def install_libs():
	from localstack_ext.packages.postgres import postgresql_package as A;from localstack_ext.services.elasticache.packages import redis_package as B;from localstack_ext.services.iot.packages import iot_rule_engine_package as C,mosquitto_package as D;from localstack_ext.services.stepfunctions.packages import stepfunctions_pro_package as E;from localstack_ext.services.timestream.packages import timescaledb_package as F
	with log_package_exception():C.install(target=InstallTarget.STATIC_LIBS)
	with log_package_exception():A.install(target=InstallTarget.STATIC_LIBS)
	with log_package_exception():F.install(target=InstallTarget.STATIC_LIBS)
	with log_package_exception():B.install(target=InstallTarget.STATIC_LIBS)
	with log_package_exception():D.install()
	E.install(target=InstallTarget.STATIC_LIBS)
def setup_ssl_cert():
	from localstack.services import generic_proxy as C;A=C.get_cert_pem_file_path()
	if os.path.exists(A):
		if is_enterprise():LOG.debug('Avoiding to update SSL certificate.');return
		D=6*60*60;E=os.path.getmtime(A)
		if E>now()-D:LOG.debug('Using cached SSL certificate (less than 6hrs since last update).');return
	LOG.debug('Attempting to download local SSL certificate file');F=3;G=5
	try:return download_github_artifact(SSL_CERT_URL,A,timeout=F)
	except Exception:
		B=SSL_CERT_URL_FALLBACK.format(api_endpoint=constants.API_ENDPOINT)
		try:return download(B,A,timeout=G)
		except Exception as H:LOG.info('Unable to download local test SSL certificate from %s to %s (using self-signed cert as fallback): %s',B,A,H);raise
def download_github_artifact(url,target_file,timeout=None):
	B=target_file;A=url
	def C(url,print_error=False):
		try:download(url,B,timeout=timeout);return True
		except Exception as A:
			if print_error:LOG.info('Unable to download Github artifact from from %s to %s: %s %s'%(url,B,A,traceback.format_exc()))
	D=C(A)
	if not D:A=A.replace('https://github.com','https://cdn.jsdelivr.net/gh');A=A.replace('/raw/master/','@master/');C(A,True)
def install_pysiddhi():from localstack_ext.services.kinesisanalytics import query_utils as A;A.setup_siddhi()
class ExtInstallerRepository(InstallerRepository):
	name='ext'
	def should_load(A):return api_key_configured()
	def load(A,*B,**C):
		LOG.debug('Preparing Pro environment for LocalStack Package Manager.')
		with prepare_environment():LOG.debug('Pro environment has successfully been prepared.')
	def get_installer(P):from localstack_ext.packages.mariadb import mariadb_package as A;from localstack_ext.packages.postgres import postgresql_package as B;from localstack_ext.services.azure.packages import azure_api_specs_package as C;from localstack_ext.services.ecr.packages import registry_package as D;from localstack_ext.services.eks.packages import k3d_package as E;from localstack_ext.services.elasticache.packages import redis_package as F;from localstack_ext.services.iot.packages import iot_rule_engine_package as G,mosquitto_package as H;from localstack_ext.services.kafka.packages import kafka_package as I;from localstack_ext.services.mq.packages import amazon_mq_package as J;from localstack_ext.services.neptune.packages import neo4j_package as K,tinkerpop_package as L;from localstack_ext.services.qldb.packages import partiql_package as M;from localstack_ext.services.stepfunctions.packages import stepfunctions_pro_package as N;from localstack_ext.services.timestream.packages import timescaledb_package as O;return[('iot-rule-engine',G),('postgres',B),('timescaledb',O),('mqtt',H),('redis',F.install),('azure',C),('ecr',D),('mysql',A),('pysiddhi',install_pysiddhi),('neo4j',K),('tinkerpop',L),('qldb',M),('k3d',E),('stepfunctions',N),('amazon-mq',J),('kafka',I)]