_D='Error while installing package %s.'
_C='install'
_B=True
_A=None
import logging,os,threading
from abc import ABC
from functools import lru_cache
from typing import List,Optional,Union
from urllib.parse import urlparse
from localstack import config
from localstack.services.install import download_and_extract_with_retry
from localstack.utils.collections import ensure_list
from localstack.utils.files import load_file,mkdir,rm_rf
from localstack.utils.platform import in_docker,is_debian
from localstack.utils.run import run
INSTALL_LOCK=threading.RLock()
POSTGRES_RPM_REPOSITORY='https://download.postgresql.org/pub/repos/yum/reporpms/EL-8-x86_64/pgdg-redhat-repo-latest.noarch.rpm'
LOG=logging.getLogger(__name__)
@lru_cache()
def is_redhat():return'rhel'in load_file('/etc/os-release','')
class PackageInstallationException(Exception):0
class SystemNotSupportedException(PackageInstallationException):0
class PackageInstaller(ABC):
	def __init__(A,log_package_name):
		B=log_package_name;A.log_package_name=B;A.logger=logging.getLogger(f"{__name__}.{B}")
		if B is _A:A.logger.setLevel(logging.CRITICAL+1)
	def install(A,raise_on_error=_B):
		try:
			if not A._check_if_available():
				try:A.logger.debug('Preparing the installation of %s.',A.log_package_name);A._prepare_installation();A.logger.debug('Starting to install %s.',A.log_package_name);A._install_package();A.logger.debug('Executing post-processing of %s.',A.log_package_name);A._post_process()
				except Exception as B:
					if isinstance(B,PackageInstallationException):raise
					else:C=f" ({A.log_package_name})."if A.log_package_name is not _A else'.';raise PackageInstallationException(f"The installation failed{C}") from B
				A._verify_installation();A.logger.debug('Successfully installed %s.',A.log_package_name)
			else:A.logger.debug('%s is already available.',A.log_package_name)
		except PackageInstallationException as B:
			if raise_on_error:raise
			elif A.logger.isEnabledFor(logging.DEBUG):A.logger.exception(_D,A.log_package_name)
			else:A.logger.error(B)
	def _check_if_available(A):raise NotImplementedError
	def _prepare_installation(A):0
	def _install_package(A):raise NotImplementedError
	def _post_process(A):0
	def _verify_installation(A):
		if not A._check_if_available():B=f" of {A.log_package_name}"if A.log_package_name is not _A else'';raise PackageInstallationException(f"The installation{B} failed (verification failed).")
class MultiPackageInstaller(PackageInstaller):
	def __init__(B,log_package_name,package_installers):A=package_installers;super(MultiPackageInstaller,B).__init__(log_package_name=log_package_name);B.package_installers=A if isinstance(A,List)else[A]
	def install(A,raise_on_error=_B):
		for B in A.package_installers:B.install(raise_on_error=raise_on_error)
class OSSpecificPackageInstaller(MultiPackageInstaller):
	def __init__(B,debian_installers,redhat_installers):
		A=[]
		if is_debian():A=debian_installers
		elif is_redhat():A=redhat_installers
		super(OSSpecificPackageInstaller,B).__init__(log_package_name=B.__class__.__name__,package_installers=A)
	def install(A,raise_on_error=_B):
		B=raise_on_error
		try:
			if not in_docker():raise SystemNotSupportedException('OS level packages are only installed within docker containers.')
			elif not is_debian()and not is_redhat():raise SystemNotSupportedException('The current operating system is currently not supported.')
			else:super(OSSpecificPackageInstaller,A).install(raise_on_error=B)
		except PackageInstallationException as C:
			if B:raise
			elif A.logger.isEnabledFor(logging.DEBUG):A.logger.exception(_D,A.log_package_name)
			else:A.logger.error(C)
class DebianPackageInstaller(PackageInstaller,ABC):
	def __init__(A,package_name):super(DebianPackageInstaller,A).__init__(log_package_name=package_name)
	def _install_os_packages(B,packages):
		A=packages;A=A if isinstance(A,List)else[A]
		with INSTALL_LOCK:C=B._download_os_packages(A);D=B.cmd_prefix(C)+[_C]+A;run(D)
	def _download_os_packages(C,packages):
		A=packages;A=ensure_list(A)
		with INSTALL_LOCK:
			D='--'.join(A);B=C.get_download_path(D);mkdir(B);run(['apt-get','update']);LOG.debug('Downloading packages %s to folder: %s',A,B)
			try:E=C.cmd_prefix(B)+['-d',_C]+A;run(E)
			except Exception as F:LOG.info('Unable to download packages %s: %s',A,F);rm_rf(B)
			return B
	def cmd_prefix(B,cache_dir):A=cache_dir;return['apt',f"-o=dir::cache={A}",f"-o=dir::cache::archives={A}",'-y']
	def get_download_path(A,package):return os.path.join(config.dirs.var_libs,'apt-libs',package)
class RedHatPackageInstaller(PackageInstaller,ABC):
	def __init__(A,package_name):super(RedHatPackageInstaller,A).__init__(log_package_name=package_name)
	def _install_os_packages(B,packages):
		A=packages;A=A if isinstance(A,List)else[A]
		with INSTALL_LOCK:run(['dnf',_C,'-y']+A)
class ArchiveDownloadInstaller(PackageInstaller):
	def __init__(A,archive_url,target_dir,cache_archive=_A,extract_single_directory=_B):
		C=cache_archive;B=archive_url;super(ArchiveDownloadInstaller,A).__init__(log_package_name=A.__class__.__name__);A.archive_url=B;A.target_dir=target_dir;A.extract_single_directory=extract_single_directory;A.cache_archive=C
		if not C:D=urlparse(B).path.split('.');E=D[-1]if len(D)>1 else'zip';A.cache_archive=f"{A.target_dir.rstrip(os.sep)}.{E}"
	def _check_if_available(A):return os.path.exists(A.target_dir)and bool(os.listdir(A.target_dir))
	def _install_package(A):
		with INSTALL_LOCK:
			A.logger.debug('Downloading and installing archive: %s',A.archive_url);download_and_extract_with_retry(A.archive_url,A.cache_archive,A.target_dir)
			if A.extract_single_directory:
				B=os.listdir(A.target_dir)
				if len(B)!=1:return
				C=os.path.join(A.target_dir,B[0])
				if not os.path.isdir(C):return
				os.rename(C,f"{A.target_dir}.bk");rm_rf(A.target_dir);os.rename(f"{A.target_dir}.bk",A.target_dir)